#include "projet.h"

extern int filtersw_opened;

#define TERR_FILTER_NOTHING_MIN 0
#define TERR_FILTER_NOTHING_MAX 0

#define TERR_FILTER_FIR_MIN 0.0
#define TERR_FILTER_FIR_MAX 1.0

#define TERR_FILTER_PUISS_MIN 0.1f
#define TERR_FILTER_PUISS_MAX 1.0f

#define TERR_FILTER_RIV_MIN 3
#define TERR_FILTER_RIV_MAX 50

#define TERR_FILTER_CRAT_MIN 1
#define TERR_FILTER_CRAT_MAX 100

static float filters_min_values[] = {
  TERR_FILTER_NOTHING_MIN,
  TERR_FILTER_FIR_MIN,
  TERR_FILTER_PUISS_MIN,
  TERR_FILTER_RIV_MIN,
  TERR_FILTER_CRAT_MIN
};

static float filters_max_values[] = {
  TERR_FILTER_NOTHING_MAX,
  TERR_FILTER_FIR_MAX,
  TERR_FILTER_PUISS_MAX,
  TERR_FILTER_RIV_MAX,
  TERR_FILTER_CRAT_MAX
};

float
get_min_for_filter (int filter)
{
  if (filter < TERR_NUM_FILTERS)
    return filters_min_values[filter];
  else
    return 0;
}

float
get_max_for_filter (int filter)
{
  if (filter < TERR_NUM_FILTERS)
    return filters_max_values[filter];
  else
    return 0;
}

void
on_filters_window_destroy (GtkObject * object, gpointer user_data)
{
  gtk_widget_destroy (GTK_WIDGET (object));
  filtersw_opened = 0;
}


void
on_fw_hscale1_value_changed (GtkRange * range, gpointer user_data)
{
  float value, max, min;
  GtkLabel *lbl =
    GTK_LABEL (lookup_widget (GTK_WIDGET (user_data), "fw_val1"));
  GtkOptionMenu *menu =
    GTK_OPTION_MENU (lookup_widget (GTK_WIDGET (user_data), "fw_option1"));
  gchar *str;

  min = get_min_for_filter (gtk_option_menu_get_history (menu));
  max = get_max_for_filter (gtk_option_menu_get_history (menu));

  value = (gtk_range_get_value (range) / 100) * (max - min) + min;
  str = g_strdup_printf ("%0.2f\n", value);
  gtk_label_set_text (lbl, str);
  g_free (str);
}


void
on_fw_hscale2_value_changed (GtkRange * range, gpointer user_data)
{
  float value, max, min;
  GtkLabel *lbl =
    GTK_LABEL (lookup_widget (GTK_WIDGET (user_data), "fw_val2"));
  GtkOptionMenu *menu =
    GTK_OPTION_MENU (lookup_widget (GTK_WIDGET (user_data), "fw_option2"));
  gchar *str;

  min = get_min_for_filter (gtk_option_menu_get_history (menu));
  max = get_max_for_filter (gtk_option_menu_get_history (menu));

  value = (gtk_range_get_value (range) / 100) * (max - min) + min;
  str = g_strdup_printf ("%0.2f\n", value);
  gtk_label_set_text (lbl, str);
  g_free (str);
}


void
on_fw_hscale3_value_changed (GtkRange * range, gpointer user_data)
{
  float value, max, min;
  GtkLabel *lbl =
    GTK_LABEL (lookup_widget (GTK_WIDGET (user_data), "fw_val3"));
  GtkOptionMenu *menu =
    GTK_OPTION_MENU (lookup_widget (GTK_WIDGET (user_data), "fw_option3"));
  gchar *str;

  min = get_min_for_filter (gtk_option_menu_get_history (menu));
  max = get_max_for_filter (gtk_option_menu_get_history (menu));

  value = (gtk_range_get_value (range) / 100) * (max - min) + min;
  str = g_strdup_printf ("%0.2f\n", value);
  gtk_label_set_text (lbl, str);
  g_free (str);
}


void
on_fw_hscale4_value_changed (GtkRange * range, gpointer user_data)
{
  float value, max, min;
  GtkLabel *lbl =
    GTK_LABEL (lookup_widget (GTK_WIDGET (user_data), "fw_val4"));
  GtkOptionMenu *menu =
    GTK_OPTION_MENU (lookup_widget (GTK_WIDGET (user_data), "fw_option4"));
  gchar *str;

  min = get_min_for_filter (gtk_option_menu_get_history (menu));
  max = get_max_for_filter (gtk_option_menu_get_history (menu));

  value = (gtk_range_get_value (range) / 100) * (max - min) + min;
  str = g_strdup_printf ("%0.2f\n", value);
  gtk_label_set_text (lbl, str);
  g_free (str);
}

void
on_menu_changed (GtkOptionMenu * menu, GtkRange * range, GtkTooltips * tips)
{
  gchar *tips_text;

  switch (gtk_option_menu_get_history (menu)) {
  case TERR_FILTER_FIR:
    tips_text =
      g_strdup_printf
      ("coefficient de lissage, entre %0.2f (pas de lissage) et %0.2f (lissage maximal)",
       TERR_FILTER_FIR_MIN, TERR_FILTER_FIR_MAX);
    break;
  case TERR_FILTER_PUISS:
    tips_text =
      g_strdup_printf ("elever a la puissance x, avec x entre %0.2f et %0.2f",
		       TERR_FILTER_PUISS_MIN, TERR_FILTER_PUISS_MAX);
    break;
  case TERR_FILTER_RIV:   
    tips_text =
      g_strdup_printf ("largeur de la riviere, entre %d et %d",
		       TERR_FILTER_RIV_MIN, TERR_FILTER_RIV_MAX);
   break;
  case TERR_FILTER_CRAT:   
    tips_text =
      g_strdup_printf ("densite de crateres, entre %d et %d",
		       TERR_FILTER_CRAT_MIN, TERR_FILTER_CRAT_MAX);
   break;
  
  default:
  case TERR_FILTER_NOTHING:
    tips_text = g_strdup_printf ("aucun parametre");
    break;
  }

  gtk_tooltips_set_tip (GTK_TOOLTIPS (tips), GTK_WIDGET (range), tips_text,
			tips_text);

  if (gtk_option_menu_get_history (menu) == TERR_FILTER_NOTHING)
    gtk_widget_set_sensitive (GTK_WIDGET (range), FALSE);
  else
    gtk_widget_set_sensitive (GTK_WIDGET (range), TRUE);

  g_free (tips_text);
}

void
on_menu1_changed (GtkOptionMenu * menu, gpointer user_data)
{
  GtkRange *range;
  GtkTooltips *tips;

  range = GTK_RANGE (lookup_widget (GTK_WIDGET (user_data), "fw_hscale1"));
  tips = GTK_TOOLTIPS (lookup_widget (GTK_WIDGET (user_data), "fw_tips1"));
  on_menu_changed (menu, range, tips);
}

void
on_menu2_changed (GtkOptionMenu * menu, gpointer user_data)
{
  GtkRange *range;
  GtkTooltips *tips;

  range = GTK_RANGE (lookup_widget (GTK_WIDGET (user_data), "fw_hscale2"));
  tips = GTK_TOOLTIPS (lookup_widget (GTK_WIDGET (user_data), "fw_tips2"));
  on_menu_changed (menu, range, tips);
}

void
on_menu3_changed (GtkOptionMenu * menu, gpointer user_data)
{
  GtkRange *range;
  GtkTooltips *tips;

  range = GTK_RANGE (lookup_widget (GTK_WIDGET (user_data), "fw_hscale3"));
  tips = GTK_TOOLTIPS (lookup_widget (GTK_WIDGET (user_data), "fw_tips3"));
  on_menu_changed (menu, range, tips);
}

void
on_menu4_changed (GtkOptionMenu * menu, gpointer user_data)
{
  GtkRange *range;
  GtkTooltips *tips;

  range = GTK_RANGE (lookup_widget (GTK_WIDGET (user_data), "fw_hscale4"));
  tips = GTK_TOOLTIPS (lookup_widget (GTK_WIDGET (user_data), "fw_tips4"));
  on_menu_changed (menu, range, tips);
}

void
on_fw_quitb_clicked (GtkButton * button, gpointer user_data)
{
  gtk_widget_destroy (GTK_WIDGET (user_data));
  filtersw_opened = 0;
}


void
on_fw_okb_clicked (GtkButton * button, gpointer user_data)
{
  GtkOptionMenu *menu;
  GtkRange *range;
  gint i = 0;
  gchar *option, *scale;
  float value, max, min;

  /* parcourt les 4 filtres */
  for (i = 1; i < 5; i++) {
    option = g_strdup_printf ("fw_option%d", i);
    scale = g_strdup_printf ("fw_hscale%d", i);

    menu = GTK_OPTION_MENU (lookup_widget (GTK_WIDGET (user_data), option));
    range = GTK_RANGE (lookup_widget (GTK_WIDGET (user_data), scale));

    min = get_min_for_filter (gtk_option_menu_get_history (menu));
    max = get_max_for_filter (gtk_option_menu_get_history (menu));

    value = (gtk_range_get_value (range) / 100) * (max - min) + min;

    /*g_print ("filter%d: %d, param=%f\n", i,
       gtk_option_menu_get_history (menu), value); */

    terrain->filters[i - 1].name = gtk_option_menu_get_history (menu);
    terrain->filters[i - 1].param = value;

    g_free (option);
    g_free (scale);
  }
  
  menu = GTK_OPTION_MENU (lookup_widget (GTK_WIDGET (user_data), "fw_colors1"));
  terrain->color_scheme = gtk_option_menu_get_history (menu);
  
  /* applique les filtres selectionns */
  ui_generic_apply_filters (terrain);
}
