/* Fichier "gtlglw_cb.c"
 * callbacks de la fenetre OpenGL de l'interface GTK+-2.x
 * 
 * Ce fichier a t crit de telle sorte qu'il ne fasse appel
 *  aucun code de rendu OpenGL _directement_ pour des raisons de 
 * portabilit vers d'autres interfaces (sait-on jamais)
 * Il ne faut appeler ici que des fonctions de gl_generic.c
 */

#include "projet.h"

int newalgow_opened = 0;
int filtersw_opened = 0;
int intro_running = 1;

static gchar *filename = NULL;
static gint already_saved = 0;
extern GtkWidget *gl_window;

extern param_hyperb_t hyp_param;
extern param_middisp_t middisp_param;
extern param_diamsquare_t diamsq_param;

void
gtkglw_activate_buttons ()
{
  GtkWidget *widg;

  /* active les boutons Refresh et Save */
  widg = lookup_widget (GTK_WIDGET (gl_window), "renderw_saveb");
  gtk_widget_set_sensitive (widg, TRUE);
  widg = lookup_widget (GTK_WIDGET (gl_window), "renderw_refrb");
  gtk_widget_set_sensitive (widg, TRUE);
  widg = lookup_widget (GTK_WIDGET (gl_window), "renderw_filtersb");
  gtk_widget_set_sensitive (widg, TRUE);
  widg = lookup_widget (GTK_WIDGET (gl_window), "enregistrer1");
  gtk_widget_set_sensitive (widg, TRUE);
  widg = lookup_widget (GTK_WIDGET (gl_window), "enregistrer_sous1");
  gtk_widget_set_sensitive (widg, TRUE);
  widg = lookup_widget (GTK_WIDGET (gl_window), "exporter1");
  gtk_widget_set_sensitive (widg, TRUE);
  widg = lookup_widget (GTK_WIDGET (gl_window), "filtres1");
  gtk_widget_set_sensitive (widg, TRUE);
  
  if (intro_running) { 
    gl_generic_init ();
    intro_running=0;
  }
}


/* ======================================================
 * on_nouveau1_activate
 *
 * Description: 
 * =====================================================
 */
void
on_nouveau1_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  GtkWidget *newalgow;

  if (!newalgow_opened) {
    newalgow = create_cal_window ();
    gtk_widget_show (newalgow);
    newalgow_opened = 1;
  }
}

void
on_quitter1_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  gl_window_idle_remove (GTK_WIDGET (user_data));
  gtk_widget_destroy (GTK_WIDGET (user_data));

  gtk_main_quit ();
}

void
callback_ouvrir_ok (GtkWidget * w, GtkFileSelection * fs)
{
  gchar *filename_open = NULL;
  void *params = NULL;
  int algo;

  filename_open =
    g_strdup (gtk_file_selection_get_filename (GTK_FILE_SELECTION (w)));

  algo = TER_read_algo (filename_open);
  if (algo < 0) {
    fprintf (stderr, "An error occured while opening terrain\n");
    /* TODO: ouvrir une boite de dialogue */
    return;
  }

  switch (algo) {
  case TERR_FLAT:
    break;
  case TERR_MIDDISP:
    params = &middisp_param;
    break;
  case TERR_SPHERE:

    break;
  case TERR_HYPERB:
    params = &hyp_param;
    break;
  case TERR_DIAM:
    params = &diamsq_param;
    break;

  default:
    fprintf (stderr, "Warning: unknown algorithm\n");
    return;
    break;
  }

  if (TER_import_terrain (terrain, params, filename_open) == 0) {
    fprintf (stderr, "An error occured while opening terrain\n");
    /* TODO: ouvrir une boite de dialogue */
    return;
  }

  if (terrain->heightmap)
    tessallate_terrain (terrain);

  else {
    fprintf (stderr, "Importation failed\n");
    return;
  }

  gtkglw_activate_buttons ();
  if (intro_running) { 
    gl_generic_init ();
    intro_running=0;
  }

  g_free (filename_open);
  gtk_widget_destroy (GTK_WIDGET (w));
}

void
on_ouvrir1_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  GtkWidget *filew;
  filew = gtk_file_selection_new (_("Ouvrir..."));

  gtk_signal_connect_object (GTK_OBJECT
			     (GTK_FILE_SELECTION (filew)->ok_button),
			     "clicked", (GtkSignalFunc) callback_ouvrir_ok,
			     filew);

  gtk_signal_connect_object (GTK_OBJECT
			     (GTK_FILE_SELECTION (filew)->cancel_button),
			     "clicked", (GtkSignalFunc) gtk_widget_destroy,
			     GTK_OBJECT (filew));

  gtk_signal_connect (GTK_OBJECT (filew), "destroy",
		      (GtkSignalFunc) gtk_widget_destroy, GTK_OBJECT (filew));

  gtk_widget_show (filew);
}

void
callback_enregsous_ok (GtkWidget * w, GtkFileSelection * fs)
{
  /*g_print ("sauvegarde de %s\n", (gchar *)
     gtk_file_selection_get_filename (GTK_FILE_SELECTION (w))); */

  if (terrain->generated) {
    if (filename)
      g_free (filename);
    filename = g_strdup (gtk_file_selection_get_filename (GTK_FILE_SELECTION
							  (w)));
    /*terr_export_png_bw (terrain, filename); */
    if (TER_export_terrain (terrain, filename) == 0) {
      fprintf (stderr, "An error occured while saving terrain\n");
      /* TODO: ouvrir une boite de dialogue */
    }
    already_saved = 1;
  }

  gtk_widget_destroy (GTK_WIDGET (w));
}

void
on_enregistrer_sous1_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  GtkWidget *filew;
  filew = gtk_file_selection_new (_("Enregistrer sous..."));
  /*gtk_file_selection_set_filename (GTK_FILE_SELECTION (filew),
     "heightmap.png"); */
  gtk_file_selection_set_filename (GTK_FILE_SELECTION (filew), "terrain.ter");

  gtk_signal_connect_object (GTK_OBJECT
			     (GTK_FILE_SELECTION (filew)->ok_button),
			     "clicked", (GtkSignalFunc) callback_enregsous_ok,
			     filew);

  gtk_signal_connect_object (GTK_OBJECT
			     (GTK_FILE_SELECTION (filew)->cancel_button),
			     "clicked", (GtkSignalFunc) gtk_widget_destroy,
			     GTK_OBJECT (filew));

  gtk_signal_connect (GTK_OBJECT (filew), "destroy",
		      (GtkSignalFunc) gtk_widget_destroy, GTK_OBJECT (filew));

  gtk_widget_show (filew);
}

void
on_enregistrer1_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  if (already_saved) {
    if (TER_export_terrain (terrain, filename) == 0) {
      fprintf (stderr, "An error occured while saving terrain\n");
      /* TODO: ouvrir une boite de dialogue */
    }
  }

  else
    on_enregistrer_sous1_activate (NULL, NULL);
}

void
callback_exporter_ok (GtkWidget * w, GtkFileSelection * fs)
{
  gchar *filename_export = NULL;

  if (terrain->generated) {

    filename_export =
      g_strdup (gtk_file_selection_get_filename (GTK_FILE_SELECTION (w)));
    if (TER_export_png_bw (terrain, filename_export) == 0) {
      fprintf (stderr, "An error occured while exporting terrain\n");
      /* TODO: ouvrir une boite de dialogue */
    }
  }

  gtk_widget_destroy (GTK_WIDGET (w));
}

void
on_exporter1_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  GtkWidget *filew;
  filew = gtk_file_selection_new (_("Exporter en PNG..."));
  gtk_file_selection_set_filename (GTK_FILE_SELECTION (filew),
				   "heightmap.png");

  gtk_signal_connect_object (GTK_OBJECT
			     (GTK_FILE_SELECTION (filew)->ok_button),
			     "clicked", (GtkSignalFunc) callback_exporter_ok,
			     filew);

  gtk_signal_connect_object (GTK_OBJECT
			     (GTK_FILE_SELECTION (filew)->cancel_button),
			     "clicked", (GtkSignalFunc) gtk_widget_destroy,
			     GTK_OBJECT (filew));

  gtk_signal_connect (GTK_OBJECT (filew), "destroy",
		      (GtkSignalFunc) gtk_widget_destroy, GTK_OBJECT (filew));

  gtk_widget_show (filew);
}

void
on_renderw_openb_clicked (GtkObject * object, gpointer user_data)
{
  on_ouvrir1_activate (NULL, NULL);
}

void
on_renderw_saveb_clicked (GtkObject * object, gpointer user_data)
{
  on_enregistrer1_activate (NULL, NULL);
}

void
on_renderw_newb_clicked (GtkObject * object, gpointer user_data)
{
  on_nouveau1_activate (NULL, NULL);
}

#define FIXME
#warning FIX ME
void
on_renderw_refrb_clicked (GtkObject * object, gpointer user_data)
{
#ifndef FIXME
  if (terrain->spec_param) {
    ui_generic_generate_new_terrain (terrain);
  }
#else
  ui_generic_generate_new_terrain (terrain);
#endif
}

void
on_about1_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  GtkWidget *aboutd = create_about_dialog ();

  gtk_widget_show (aboutd);
}

void
on_filtres1_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  GtkWidget *filterw;

  if (!filtersw_opened) {
    filterw = create_filters_window ();
    gtk_widget_show (filterw);
    filtersw_opened = 1;
  }
}


void
on_renderw_filtersb_clicked (GtkObject * object, gpointer user_data)
{
  on_filtres1_activate (NULL, NULL);
}

void
on_backfaceculling1_activate (GtkCheckMenuItem * menuitem, gpointer user_data)
{
  gl_generic_switch_bfculling ();
}

void
on_wireframe1_activate (GtkCheckMenuItem * menuitem, gpointer user_data)
{
  gl_generic_switch_wireframe ();
}

void
on_terrain1_activate (GtkRadioMenuItem * menuitem, gpointer user_data)
{
  cam_set_control_mode (CAMERA_CONTROL_TERRAIN);
}

void
on_camera1_activate (GtkRadioMenuItem * menuitem, gpointer user_data)
{
  cam_set_control_mode (CAMERA_CONTROL_CAMERA);
}


/* ======================================================
 * on_gl_window_destroy
 *
 * Description: appele quand le signal "destroy" est recu
 *  sur la fenetre GtkGLExt.
 * =====================================================
 */
void
on_gl_window_destroy (GtkObject * object, gpointer user_data)
{
  gl_window_idle_remove (GTK_WIDGET (user_data));
  gtk_widget_destroy (GTK_WIDGET (user_data));

  gtk_main_quit ();
}

/* ======================================================
 * on_gl_window_draw
 *
 * Description: appele quand le signal "expose_event" est recu
 *  sur la fenetre GtkGLExt.
 * =====================================================
 */
gboolean
on_gl_window_draw (GtkWidget * widget, GdkEventExpose * event, gpointer data)
{
  GdkGLContext *glcontext = gtk_widget_get_gl_context (widget);
  GdkGLDrawable *gldrawable = gtk_widget_get_gl_drawable (widget);

  /*** OpenGL BEGIN ***/
  if (!gdk_gl_drawable_gl_begin (gldrawable, glcontext))
    return FALSE;

  if (!intro_running)
    gl_generic_render_scene ();
  
  else glintro_draw_2D_intro ();

  if (gdk_gl_drawable_is_double_buffered (gldrawable))
    gdk_gl_drawable_swap_buffers (gldrawable);
  else
    glFlush ();

  gdk_gl_drawable_gl_end (gldrawable);
  /*** OpenGL END ***/

  return TRUE;
}

/* ======================================================
 * on_gl_window_reshape
 *
 * Description: appele quand le signal "configure_event" est recu
 *  sur la fenetre GtkGLExt, i.e. quand on redimensionne,
 *  ou quand la fenetre apparait apres avoir t cache
 * =====================================================
 */
gboolean
on_gl_window_reshape (GtkWidget * widget,
		      GdkEventConfigure * event, gpointer data)
{
  GdkGLContext *glcontext = gtk_widget_get_gl_context (widget);
  GdkGLDrawable *gldrawable = gtk_widget_get_gl_drawable (widget);

  /*** OpenGL BEGIN ***/
  if (!gdk_gl_drawable_gl_begin (gldrawable, glcontext))
    return FALSE;

  if (!intro_running)
    gl_generic_resize (widget->allocation.width, widget->allocation.height);

  gdk_gl_drawable_gl_end (gldrawable);
  /*** OpenGL END ***/

  return TRUE;
}


/* ======================================================
 * on_gl_window_init
 *
 * Description: appele quand le signal "realize" est recu
 *  sur la fenetre GtkGLExt, i.e. quand la fenetre est cree.
 * =====================================================
 */
void
on_gl_window_init (GtkWidget * widget, gpointer data)
{
  GdkGLContext *glcontext = gtk_widget_get_gl_context (widget);
  GdkGLDrawable *gldrawable = gtk_widget_get_gl_drawable (widget);

  /*** OpenGL BEGIN ***/
  if (!gdk_gl_drawable_gl_begin (gldrawable, glcontext))
    return;

  gl_generic_init_2D_intro ();

  gdk_gl_drawable_gl_end (gldrawable);
  /*** OpenGL END ***/
}

/* ======================================================
 * on_gl_window_idle
 *
 * Description: appele quand on n'a rien d'autre  faire.
 * =====================================================
 */
gboolean
on_gl_window_idle (GtkWidget * widget)
{
  gtk_widget_queue_draw (widget);

  return TRUE;
}

static guint idle_id = 0;

/* ======================================================
 * gl_window_idle_add
 *
 * Description:
 * =====================================================
 */
void
gl_window_idle_add (GtkWidget * widget)
{
  if (idle_id == 0) {
    idle_id = gtk_idle_add_priority (GDK_PRIORITY_REDRAW,
				     (GtkFunction) on_gl_window_idle, widget);
  }
}

/* ======================================================
 * gl_window_idle_remove
 *
 * Description:
 * =====================================================
 */
void
gl_window_idle_remove (GtkWidget * widget)
{
  if (idle_id != 0) {
    gtk_idle_remove (idle_id);
    idle_id = 0;
  }
}

/* ======================================================
 * on_gl_window_map
 *
 * Description:
 * =====================================================
 */
gboolean
on_gl_window_map (GtkWidget * widget, GdkEventAny * event, gpointer data)
{
  gl_window_idle_add (widget);

  return TRUE;
}

/* ======================================================
 * on_gl_window_unmap
 *
 * Description:
 * =====================================================
 */
gboolean
on_gl_window_unmap (GtkWidget * widget, GdkEventAny * event, gpointer data)
{
  gl_window_idle_remove (widget);

  return TRUE;
}

/* ======================================================
 * on_gl_window_visible
 *
 * Description: appele quand le signal "visibility_notify_event" est recu
 *  sur la fenetre GtkGLExt, i.e. quand il y a un changement
 *  de visibilit de la fenetre (partiellement recouverte etc...)
 * =====================================================
 */
gboolean
on_gl_window_visible (GtkWidget * widget,
		      GdkEventVisibility * event, gpointer data)
{
  if (event->state == GDK_VISIBILITY_FULLY_OBSCURED)
    gl_window_idle_remove (widget);
  else
    gl_window_idle_add (widget);

  return TRUE;
}

/* ======================================================
 * on_gl_window_key
 *
 * Description: appele quand le signal "key_press_event" est recu
 *  sur la fenetre GtkGLExt, i.e. quand on appuie sur une touche
 * =====================================================
 */
gboolean
on_gl_window_key (GtkWidget * widget, GdkEventKey * event, gpointer data)
{
  if (cam_get_control_mode () == CAMERA_CONTROL_TERRAIN) {
    switch (event->keyval) {
    case GDK_Up:
      cam_move_cam_forwbackw (2);
      break;
    case GDK_Down:
      cam_move_cam_forwbackw (-2);
      break;
    case GDK_Left:
      cam_rotate_terrain_leftright (0.1);
      break;
    case GDK_Right:
      cam_rotate_terrain_leftright (-0.1);
      break;
    case GDK_Page_Up:
      cam_rotate_cam_updown (-0.03);
      break;
    case GDK_Page_Down:
      cam_rotate_cam_updown(0.03);
      break;
    case GDK_Escape:
      cam_set_default_view ();
      break;
    case GDK_w:
      gl_generic_switch_wireframe ();
      break;
    default:
      return FALSE;
    }
  }

  else {
    switch (event->keyval) {
    case GDK_Up:
      cam_move_cam_forwbackw (2);
      break;
    case GDK_Down:
      cam_move_cam_forwbackw (-2);
      break;
    case GDK_Left:
      cam_rotate_cam_leftright (0.05);
      break;
    case GDK_Right:
      cam_rotate_cam_leftright (-0.05); 
      break;
    case GDK_q:
      cam_strafe_cam_leftright (-2);
      break;
    case GDK_d:
      cam_strafe_cam_leftright (2);
      break;
    case GDK_z:
      cam_move_cam_updown (2);
      break;
    case GDK_s:
      cam_move_cam_updown (-2);
      break;
    case GDK_Page_Up:
      cam_rotate_cam_updown (-0.03);
      break;
    case GDK_Page_Down:
      cam_rotate_cam_updown(0.03);
      break;
    case GDK_Escape:
      cam_set_default_view ();
      break;
    case GDK_w:
      gl_generic_switch_wireframe ();
      break;
    default:
      return FALSE;
    }
  }

  gtk_widget_queue_draw (widget);

  return TRUE;
}
